classdef AaroniaHTTP
    properties
        address
    end
   methods
        function obj = AaroniaHTTP(address)
            obj.address = address;
        end
       
        function [IQData] = getIQFromPacket(obj, packet)
                if(~isempty(packet))
                    I = packet.samples(1:2:end);
                    Q = packet.samples(2:2:end);
                    IQData = I+1i*Q;
                else
                    IQData = [];
                end
        end

        function [packet, statusCode] = getSamplePacket(obj)
            import matlab.net.*
            import matlab.net.http.*

            r = RequestMessage;
            resp = send(r,URI(append(obj.address , 'sample')));

            if(isempty(resp.Body.Data))
                disp('No Data received');
            else
                disp('Packet received:');
                disp( resp.Body.Data );
            end

            packet = resp.Body.Data;
            statusCode = resp.StatusCode;
        end

        function [packets, statusCode] = getStreamPacket(obj, limit)
            import matlab.net.*
            import matlab.net.http.*

            r = RequestMessage;
            resp = send(r,URI(sprintf("%sstream?limit=%d&format=float32", obj.address, limit)));
            packet = resp.Body.Data;

            packets = [];

            while size(packet) > 0
                idx = find(packet == 0x1e, 1);
                metadata = jsondecode(convertCharsToStrings(native2unicode(packet(1:(idx-2)))));
                sampleData = packet(idx+1:(idx+4*metadata.sampleSize*metadata.samples));
                interleavedSamples = typecast(sampleData,'single');
                samples = interleavedSamples(1:2:size(interleavedSamples)) + 1j * interleavedSamples(2:2:size(interleavedSamples));

                metadata.iqData = samples;

                packets = [ packets metadata ];
                packet = packet((idx+1+4*metadata.sampleSize*metadata.samples):size(packet));
            end
            
            statusCode = resp.StatusCode;
        end

        function [remoteConfig, statusCode] = getRemoteConfig(obj)
            import matlab.net.*
            import matlab.net.http.*

            r = RequestMessage;

            resp = send(r , URI(append(obj.address , 'remoteconfig')));
            remoteConfig = resp.Body.Data.config;
            statusCode = resp.StatusCode;
        end
        
        function [response, statusCode] = setFileParameter(obj, blockName, varGroup, varName, varValue)

            item(1).type = 'file';
            item(1).name = varName;
            item(1).file = varValue;

            if(~isempty(varGroup))
                items(1).name = varGroup;
                items(1).items = { item } ;
            else
                items = item;
            end

            [response, statusCode] = setParameter( obj, blockName, items );
        end
        function [response, statusCode] = setEnumParameter(obj, blockName, varGroup, varName, varValue, varValues)

            item(1).type = 'enum';
            item(1).name = varName;
            item(1).value = varValue;
            item(1).values = varValues;

            if(~isempty(varGroup))
                items(1).name = varGroup;
                items(1).items = { item } ;
            else
                items = item;
            end

            [response, statusCode] = setParameter( obj, blockName, items );
        end   
        function [response, statusCode] = setValueParameter(obj, blockName, varGroup, varType, varName, varValue)

            item(1).type = varType;
            item(1).name = varName;
            item(1).value = varValue;

            if(~isempty(varGroup))
                items(1).name = varGroup;
                items(1).items = { item } ;
            else
                items = item;
            end

            [response, statusCode] = setParameter( obj, blockName, items );
        end
        
        function [response, statusCode] = setParameter(obj, blockName, items)

            block(1).name = blockName;
            block(1).items = { items };

            config = struct('config',{});
            config(1).config = block;

            [response, statusCode] = putRequest(obj, append(obj.address , 'remoteconfig'), jsonencode(config));
        end
        
        function [statusCode] = startStreaming(obj)
            config(1).start = true;
            config(1).type = 'streaming';

            [~, statusCode] = putRequest(obj, append(obj.address , 'control'), jsonencode(config));
        end

        function [statusCode] = StopStreaming(obj)
            config(1).start = false;
            config(1).type = 'streaming';

            [~, statusCode] = putRequest(obj, append(obj.address , 'control'), jsonencode(config));
        end
        
        function [healthstatus, statusCode] = getHealthStatus(obj)
            import matlab.net.*
            import matlab.net.http.*

            r = RequestMessage;

            resp = send(r , URI(append(obj.address , 'healthstatus')));
            disp(resp);
            healthstatus = resp.Body.Data;
            statusCode = resp.StatusCode;
        end
       
        function [response, statusCode] = putRequest(obj, uri, data)
            import matlab.net.*
            import matlab.net.http.*

            body = matlab.net.http.MessageBody(data);

            contentTypeField = matlab.net.http.field.ContentTypeField...
                ('application/x-www-form-urlencoded');
            acceptField = matlab.net.http.field.AcceptField...
                ( matlab.net.http.MediaType('*/*') );
            header = [acceptField contentTypeField];

            request = matlab.net.http.RequestMessage(...
                matlab.net.http.RequestMethod.PUT...
                ,header,...
                body);

            show(request);

            response = send( request , URI(uri) );
            disp(response);
            statusCode = response.StatusCode;
        end
   end
end

